# Copyright 2021 Sushil Khanchi <sushilkhanchi97@gmail.com>
# Copyright 2022 Georges Racinet <georges.racinet@octobus.net>
#
# This software may be used and distributed according to the terms of the
# GNU General Public License version 2 or any later version.
#
# SPDX-License-Identifier: GPL-2.0-or-later
"""Utilities to generate Mercurial revset expressions.

Notably used to convert Git revision specifications, see the
gitrevisions(7) man page.
"""

from .revision import (
    gitlab_revision_changeset,
)


class FollowNotImplemented(NotImplementedError):
    pass


def revset_from_git_revspec(repo, revspec, for_follow=False):
    """Generate Mercurial revset from a given Git revspec.

    In many cases, `revision` arguments to Gitaly gRPC methods can be
    any valid Git revision specifications (revspecs), since they often are
    simply forwarded to an actual Git subprocess.

    At this point, we only support most common Git revision ranges, and
    the case of a single revision.

    No symbolic name like bookmarks, GitLab branches or tags are in the
    resulting revset: there are only operators and revnos or node ids.
    Hence it is appropriate (and often needed) to use it on unfiltered
    repositories.

    The resulting revset includes the sorting directive to make it consistent
    with the default `git log` ordering.

    :param bytes revspec: Git revision specification
    :param for_follow: if ``True``, indicates that the caller intends to use
       the resulting revset with additional following of copy tracking. In
       that case, specifying ancestors is to be avoided, as it would lead
       to iterate quadratically on them (ancestors of each ancestor), which
       can exhaust all system resources on large repositories (hgitaly#117).
    :return: the resulting Mercurial `revset`, or ``None`` if not found.
    """
    if b'...' in revspec:
        r1, r2 = revspec.split(b'...')
        revset = revset_symmetric_difference(repo, r1, r2,
                                             for_follow=for_follow)
    elif b'..' in revspec:
        r1, r2 = revspec.split(b'..')
        revset = revset_git_range(repo, r1, r2, for_follow=for_follow)

    else:  # single revision
        ctx = gitlab_revision_changeset(repo, revspec)
        if ctx is None:
            return None
        return ctx.hex() if for_follow else b"reverse(::%s)" % ctx

    return b"sort(%s, -rev)" % revset


def revset_git_range(repo, r1, r2, for_follow=False):
    """Make a revset for a Git (two-dot) Range Notation

    Remind that the Dotted Range Notations are just notations looking like
    ranges, but do not express actual DAG ranges.

    Excerpt from gitrevisions(7):

      The ``..`` (two-dot) Range Notation
        The ``^r1 r2`` set operation appears so often that there is a
        shorthand for it. When you have two commits ``r1`` and ``r2`` (named
        according to the syntax explained in SPECIFYING REVISIONS above),
        you can ask for commits that are reachable from ``r2`` excluding those
        that are reachable from ``r1`` by ``^r1 r2`` and it can be written as
        ``r1..r2``.

      (...)

      In these [two] shorthand notations, you can omit one end and let it
      default to HEAD.

    :param for_follow: same meaning as in :`func`:`revset_from_git_revspec`
    """
    if for_follow:
        raise FollowNotImplemented()

    ctx_start, ctx_end = resolve_git_range(repo, r1, r2)
    return b"::%s - ::%s" % (ctx_end, ctx_start)


def revset_symmetric_difference(repo, r1, r2, for_follow=False):
    """Make a revset for a Git Symmetric Difference.

    Excerpt from gitrevisions(7):

      The ``...`` (three-dot) Symmetric Difference Notation
          A similar notation ``r1...r2`` is called symmetric difference of
          ``r1`` and ``r2`` and is defined as
          ``r1 r2 --not $(git merge-base --all r1 r2)``.
          It is the set of commits that are reachable from either one of
          ``r1`` (left side) or ``r2`` (right side) but  not from both.

      In these [two] shorthand notations, you can omit one end and let it
      default to HEAD.

    :param for_follow: same meaning as in :`func`:`revset_from_git_revspec`
    """
    if for_follow:
        raise FollowNotImplemented()

    ctx_start, ctx_end = resolve_git_range(repo, r1, r2)
    left = ctx_start.rev()
    right = ctx_end.rev()
    branchpoint = repo.revs(b"ancestor(%d, %d)" % (left, right)).first()
    return b"only(%d + %d, %d)" % (left, right, branchpoint)


def resolve_git_range(repo, r1, r2):
    """Resolve the two ends of a Git Dotted Range Notation.

    Remind that the Dotted Range Notations are just notations looking like
    ranges, but do not express actual DAG ranges.

    :return: a pair of changeset contexts

    Excerpt from gitrevisions(7):

      In these two shorthand notations (two and three dot range notations),
      you can omit one end and let it default to ``HEAD``. For example,
      ``origin..`` is a shorthand for ``origin..HEAD`` and asks "What did I do
      since I forked from the ``origin`` branch?" Similarly, ``..origin`` is
      a shorthand for ``HEAD..origin`` and asks "What did the origin do since
      I forked from them?"
    """
    if not r1:
        r1 = b'HEAD'
    if not r2:
        r2 = b'HEAD'
    return tuple(gitlab_revision_changeset(repo, r) for r in (r1, r2))


def changeset_descr_regexp(pattern, ignore_case=False):
    """Generate a revset regular expression for changeset description.

    This takes care of any necessary escaping.
    """
    predicate = b'hpd_dsc_irx' if ignore_case else b'hpd_dsc_rx'
    return b"%s(r'%s')" % (predicate, pattern)



# Copyright 2020 Georges Racinet <georges.racinet@octobus.net>
#
# This software may be used and distributed according to the terms of the
# GNU General Public License version 2 or any later version.
#
# SPDX-License-Identifier: GPL-2.0-or-later
"""Git specific things, mostly constants."""

import fnmatch

ZERO_SHA_1 = '0' * 40

NULL_COMMIT_ID = ZERO_SHA_1
NULL_BLOB_OID = ZERO_SHA_1

# from `sha1-file.c` in Git 2.28 sources
# we're not dealing for now with the fact that there will be
# two kinds of OIDs with SHA-1 and SHA-256 soon.

# The Git tree object hash that corresponds to an empty tree (directory)
EMPTY_TREE_OID = '4b825dc642cb6eb9a060e54bf8d69288fbee4904'

# The Git blob object hash that corresponds to an empty blob (file)
EMPTY_BLOB_OID = 'e69de29bb2d1d6434b8b29ae775ad8c2e48c5391'

OBJECT_MODE_DOES_NOT_EXIST = 0  # see, e.g, ChangedPaths in diff.proto
OBJECT_MODE_LINK = 0o120000  # symlink to file or directory
OBJECT_MODE_EXECUTABLE = 0o100755  # for blobs only
OBJECT_MODE_NON_EXECUTABLE = 0o100644  # for blobs only
OBJECT_MODE_TREE = 0o40000

FILECTX_FLAGS_TO_GIT_MODE_BYTES = {
    b'l': b'%o' % OBJECT_MODE_LINK,
    b'x': b'%o' % OBJECT_MODE_EXECUTABLE,
    b'': b'%o' % OBJECT_MODE_NON_EXECUTABLE,
}


class GitPathSpec:
    """File path matching as Git pathspecs do.

    Reference: gitglossary(7)

    This implementation is to be completed (magic words etc.

    Example from the glossary about wildcards matching slashes::

      >>> pathspec = GitPathSpec(b'Documentation/*.jpg')
      >>> pathspec.match(b'Documentation/chapter_1/figure_1.jpg')
      True

    Before the last slash, wildcards do not match slashes::

      >>> pathspec = GitPathSpec(b'*/figure_1.jpg')
      >>> pathspec.match(b'Documentation/chapter_1/figure_1.jpg')
      False
      >>> pathspec.match(b'figure_1.jpg')
      False
      >>> pathspec.match(b'Documentation/figure_1.jpg')
      True

    Trailing slash subtleties::

      >>> pathspec = GitPathSpec(b'Documentation/')
      >>> pathspec.match(b'Documentation')
      False
      >>> pathspec.match(b'Documentation/foo')
      True

    Implicit directory prefix (not mentioned in gitglossary(7) but still
    true and checkable with all commands, including `git log` and `git add`)::

      >>> pathspec = GitPathSpec(b'Documentation/foo')
      >>> pathspec.match(b'Documentation/foo/bar')
      True

      >>> pathspec = GitPathSpec(b'Documentation')
      >>> pathspec.match(b'Documentation')
      True
      >>> pathspec.match(b'Documentation/foo')
      True
      >>> pathspec.match(b'Documentation/foo/bar')
      True

      >>> pathspec = GitPathSpec(b'Doc*')
      >>> pathspec.match(b'Documentation/foo')
      True
      >>> pathspec.match(b'Documentation/foo/bar')
      True
    """

    def __init__(self, pathspec: bytes):
        self.split_spec = pathspec.split(b'/')
        # Note that `fnmatch.translate()` (outputs regexp) chokes
        # on bytes strings.

    def match(self, file_path):
        split_input = file_path.split(b'/', len(self.split_spec) - 1)
        if len(split_input) < len(self.split_spec):
            return False

        split_spec_but_one = self.split_spec[:-1]
        # a split is never empty, even if done for the empty string
        last_spec = self.split_spec[-1]

        split_input_but_one = split_input[:-1]
        last_input = split_input[-1]

        # all elements of both lists except the last one are free of slashes,
        # hence fnmatch cannot create a deeper match for patterns before
        # the last slash in pathspec.
        return (all(not s or fnmatch.fnmatch(i, s)
                    for i, s in zip(split_input_but_one, split_spec_but_one))
                and
                (
                    # trailing slash in pathspec
                    not last_spec
                    # case of deep fnmatch accross remaining directories
                    or fnmatch.fnmatch(last_input, last_spec)
                    # case of pattern=foo and input=foo/bar
                    or fnmatch.fnmatch(last_input.split(b'/', 1)[0], last_spec)
                ))


class GitLiteralPathSpec:
    def __init__(self, pathspec):
        self.pathspec = pathspec
        if pathspec.endswith(b'/'):
            self.dir_prefix = pathspec
        else:
            self.dir_prefix = pathspec + b'/'

    def match(self, file_path):
        return (
            file_path == self.pathspec
            or file_path.startswith(self.dir_prefix)
        )



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc

from . import analysis_pb2 as analysis__pb2


class AnalysisServiceStub(object):
    """AnalysisService is a gRPC service providing RPCs that analyze objects in a repository.
    """

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.CheckBlobsGenerated = channel.stream_stream(
                '/gitaly.AnalysisService/CheckBlobsGenerated',
                request_serializer=analysis__pb2.CheckBlobsGeneratedRequest.SerializeToString,
                response_deserializer=analysis__pb2.CheckBlobsGeneratedResponse.FromString,
                )


class AnalysisServiceServicer(object):
    """AnalysisService is a gRPC service providing RPCs that analyze objects in a repository.
    """

    def CheckBlobsGenerated(self, request_iterator, context):
        """CheckBlobsGenerated checks a provided set of blobs in a specified repository to determine
        whether the file is considered generated. This RPC supports bidirectional streaming because the
        client may specify any number of files to check across multiple request messages and the server
        responds to each request with a separate set of response messages.

        Each provided blob in the request is validated when received. Improperly formatted requests
        result in RPC termination and an error being returned to the client. The RPC also aborts and
        returns an error if requests are made that list Git revisions which do not resolve to a blob
        and/or cannot be found in the repository.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_AnalysisServiceServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'CheckBlobsGenerated': grpc.stream_stream_rpc_method_handler(
                    servicer.CheckBlobsGenerated,
                    request_deserializer=analysis__pb2.CheckBlobsGeneratedRequest.FromString,
                    response_serializer=analysis__pb2.CheckBlobsGeneratedResponse.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'gitaly.AnalysisService', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class AnalysisService(object):
    """AnalysisService is a gRPC service providing RPCs that analyze objects in a repository.
    """

    @staticmethod
    def CheckBlobsGenerated(request_iterator,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.stream_stream(request_iterator, target, '/gitaly.AnalysisService/CheckBlobsGenerated',
            analysis__pb2.CheckBlobsGeneratedRequest.SerializeToString,
            analysis__pb2.CheckBlobsGeneratedResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)



# Copyright 2022 Georges Racinet <georges.racinet@octobus.net>
#
# This software may be used and distributed according to the terms of the
# GNU General Public License version 2 or any later version.
#
# SPDX-License-Identifier: GPL-2.0-or-later
import pytest

from ..revset import (
    FollowNotImplemented,
    revset_from_git_revspec,
)


def test_follow_not_implemented():
    # we don't even need to pass a true repo or revisions to test this
    with pytest.raises(FollowNotImplemented):
        revset_from_git_revspec(None, b'foo..bar', for_follow=True)

    with pytest.raises(FollowNotImplemented):
        revset_from_git_revspec(None, b'foo...bar', for_follow=True)



# Copyright 2021 Georges Racinet <georges.racinet@octobus.net>
#
# This software may be used and distributed according to the terms of the
# GNU General Public License version 2 or any later version.
#
# SPDX-License-Identifier: GPL-2.0-or-later

from heptapod.testhelpers import (
    LocalRepoWrapper,
)
from ..file_context import (
    git_perms
)
from .. import git


def make_repo(path):
    return LocalRepoWrapper.init(path,
                                 config=dict(
                                     extensions=dict(topic='', evolve=''),
                                 ))


def test_git_perms(tmpdir):
    wrapper = make_repo(tmpdir)
    regular = (tmpdir / 'regular')
    regular.write('foo')

    script = tmpdir / 'script'
    script.write('#!/usr/bin/env python2\n'
                 'print "Hello, world"\n')
    script.chmod(0o755)

    (tmpdir / 'link_exe').mksymlinkto(script)
    (tmpdir / 'link_regular').mksymlinkto(regular)

    changeset = wrapper.commit([], add_remove=True)
    assert git_perms(changeset[b'script']) == git.OBJECT_MODE_EXECUTABLE
    assert git_perms(changeset[b'regular']) == git.OBJECT_MODE_NON_EXECUTABLE
    assert git_perms(changeset[b'link_regular']) == git.OBJECT_MODE_LINK
    # gracinet: I checked manually with a local repo that symlinks to
    # executable and non executable files have the same Git file mode.
    assert git_perms(changeset[b'link_exe']) == git.OBJECT_MODE_LINK



# Copyright 2020 Georges Racinet <georges.racinet@octobus.net>
#
# This software may be used and distributed according to the terms of the
# GNU General Public License version 2 or any later version.
#
# SPDX-License-Identifier: GPL-2.0-or-later
import re
from pkg_resources import parse_version

from hgitaly.stub.server_pb2 import (
    ServerInfoRequest,
)
from hgitaly.stub.server_pb2_grpc import ServerServiceStub


def test_server_info(grpc_channel):
    grpc_stub = ServerServiceStub(grpc_channel)

    resp = grpc_stub.ServerInfo(ServerInfoRequest())
    version = resp.server_version
    assert version
    assert re.match(r'\d+[.]\d+[.]\d+',
                    parse_version(version).base_version) is not None



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: repository.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from . import lint_pb2 as lint__pb2
from . import shared_pb2 as shared__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x10repository.proto\x12\x06gitaly\x1a\nlint.proto\x1a\x0cshared.proto\"G\n\x17RepositoryExistsRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\"*\n\x18RepositoryExistsResponse\x12\x0e\n\x06\x65xists\x18\x01 \x01(\x08\"E\n\x15RepositorySizeRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\"&\n\x16RepositorySizeResponse\x12\x0c\n\x04size\x18\x01 \x01(\x03\"E\n\x15RepositoryInfoRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\"\xbb\x02\n\x16RepositoryInfoResponse\x12\x0c\n\x04size\x18\x01 \x01(\x04\x12\x41\n\nreferences\x18\x02 \x01(\x0b\x32-.gitaly.RepositoryInfoResponse.ReferencesInfo\x12;\n\x07objects\x18\x03 \x01(\x0b\x32*.gitaly.RepositoryInfoResponse.ObjectsInfo\x1a:\n\x0eReferencesInfo\x12\x13\n\x0bloose_count\x18\x01 \x01(\x04\x12\x13\n\x0bpacked_size\x18\x02 \x01(\x04\x1aW\n\x0bObjectsInfo\x12\x0c\n\x04size\x18\x01 \x01(\x04\x12\x13\n\x0brecent_size\x18\x02 \x01(\x04\x12\x12\n\nstale_size\x18\x03 \x01(\x04\x12\x11\n\tkeep_size\x18\x04 \x01(\x04\"U\n\x12ObjectsSizeRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\x12\x11\n\trevisions\x18\x02 \x03(\x0c\"#\n\x13ObjectsSizeResponse\x12\x0c\n\x04size\x18\x01 \x01(\x04\"C\n\x13ObjectFormatRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\"<\n\x14ObjectFormatResponse\x12$\n\x06\x66ormat\x18\x01 \x01(\x0e\x32\x14.gitaly.ObjectFormat\"[\n\x19\x41pplyGitattributesRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\x12\x10\n\x08revision\x18\x02 \x01(\x0c\"\x1c\n\x1a\x41pplyGitattributesResponse\"e\n\x12\x46\x65tchBundleRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\x12\x0c\n\x04\x64\x61ta\x18\x02 \x01(\x0c\x12\x13\n\x0bupdate_head\x18\x03 \x01(\x08\"\x15\n\x13\x46\x65tchBundleResponse\"\x82\x02\n\x12\x46\x65tchRemoteRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\x12\r\n\x05\x66orce\x18\x03 \x01(\x08\x12\x0f\n\x07no_tags\x18\x04 \x01(\x08\x12\x0f\n\x07timeout\x18\x05 \x01(\x05\x12\x0f\n\x07ssh_key\x18\x06 \x01(\t\x12\x13\n\x0bknown_hosts\x18\x07 \x01(\t\x12\x10\n\x08no_prune\x18\t \x01(\x08\x12%\n\rremote_params\x18\n \x01(\x0b\x32\x0e.gitaly.Remote\x12\x1a\n\x12\x63heck_tags_changed\x18\x0b \x01(\x08J\x04\x08\x08\x10\tJ\x04\x08\x02\x10\x03R\x06remote\"+\n\x13\x46\x65tchRemoteResponse\x12\x14\n\x0ctags_changed\x18\x01 \x01(\x08\"\x8c\x01\n\x17\x43reateRepositoryRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\x12\x16\n\x0e\x64\x65\x66\x61ult_branch\x18\x02 \x01(\x0c\x12+\n\robject_format\x18\x03 \x01(\x0e\x32\x14.gitaly.ObjectFormat\"\x1a\n\x18\x43reateRepositoryResponse\"\x99\x02\n\x11GetArchiveRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\x12\x11\n\tcommit_id\x18\x02 \x01(\t\x12\x0e\n\x06prefix\x18\x03 \x01(\t\x12\x30\n\x06\x66ormat\x18\x04 \x01(\x0e\x32 .gitaly.GetArchiveRequest.Format\x12\x0c\n\x04path\x18\x05 \x01(\x0c\x12\x0f\n\x07\x65xclude\x18\x06 \x03(\x0c\x12\x12\n\nelide_path\x18\x07 \x01(\x08\x12\x19\n\x11include_lfs_blobs\x18\x08 \x01(\x08\"3\n\x06\x46ormat\x12\x07\n\x03ZIP\x10\x00\x12\x07\n\x03TAR\x10\x01\x12\n\n\x06TAR_GZ\x10\x02\x12\x0b\n\x07TAR_BZ2\x10\x03\"\"\n\x12GetArchiveResponse\x12\x0c\n\x04\x64\x61ta\x18\x01 \x01(\x0c\"G\n\x17HasLocalBranchesRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\")\n\x18HasLocalBranchesResponse\x12\r\n\x05value\x18\x01 \x01(\x08\"\xc3\x01\n\x18\x46\x65tchSourceBranchRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\x12-\n\x11source_repository\x18\x02 \x01(\x0b\x32\x12.gitaly.Repository\x12\x15\n\rsource_branch\x18\x03 \x01(\x0c\x12\x12\n\ntarget_ref\x18\x04 \x01(\x0c\x12\x1f\n\x17\x65xpected_target_old_oid\x18\x05 \x01(\t\"+\n\x19\x46\x65tchSourceBranchResponse\x12\x0e\n\x06result\x18\x01 \x01(\x08\";\n\x0b\x46sckRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\"\x1d\n\x0c\x46sckResponse\x12\r\n\x05\x65rror\x18\x01 \x01(\x0c\"\x87\x01\n\x0fWriteRefRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\x12\x0b\n\x03ref\x18\x02 \x01(\x0c\x12\x10\n\x08revision\x18\x03 \x01(\x0c\x12\x14\n\x0cold_revision\x18\x04 \x01(\x0cJ\x04\x08\x05\x10\x06J\x04\x08\x06\x10\x07R\x05\x66orce\"\x18\n\x10WriteRefResponseJ\x04\x08\x01\x10\x02\"W\n\x14\x46indMergeBaseRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\x12\x11\n\trevisions\x18\x02 \x03(\x0c\"%\n\x15\x46indMergeBaseResponse\x12\x0c\n\x04\x62\x61se\x18\x01 \x01(\t\"\x82\x01\n\x11\x43reateForkRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\x12-\n\x11source_repository\x18\x02 \x01(\x0b\x32\x12.gitaly.Repository\x12\x10\n\x08revision\x18\x03 \x01(\x0c\"\x14\n\x12\x43reateForkResponse\"\xb9\x01\n\x1e\x43reateRepositoryFromURLRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\x12\x0b\n\x03url\x18\x02 \x01(\t\x12!\n\x19http_authorization_header\x18\x04 \x01(\t\x12\x0e\n\x06mirror\x18\x05 \x01(\x08\x12\x18\n\x10resolved_address\x18\x06 \x01(\tJ\x04\x08\x03\x10\x04R\thttp_host\"!\n\x1f\x43reateRepositoryFromURLResponse\"C\n\x13\x43reateBundleRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\"$\n\x14\x43reateBundleResponse\x12\x0c\n\x04\x64\x61ta\x18\x01 \x01(\x0c\"`\n\x1e\x43reateBundleFromRefListRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\x12\x10\n\x08patterns\x18\x02 \x03(\x0c\"/\n\x1f\x43reateBundleFromRefListResponse\x12\x0c\n\x04\x64\x61ta\x18\x01 \x01(\x0c\"H\n\x18GenerateBundleURIRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\"\x1b\n\x19GenerateBundleURIResponse\"@\n\x10GetConfigRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\"!\n\x11GetConfigResponse\x12\x0c\n\x04\x64\x61ta\x18\x01 \x01(\x0c\"W\n\x19RestoreCustomHooksRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\x12\x0c\n\x04\x64\x61ta\x18\x02 \x01(\x0c\"S\n\x15SetCustomHooksRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\x12\x0c\n\x04\x64\x61ta\x18\x02 \x01(\x0c\"\x1c\n\x1aRestoreCustomHooksResponse\"\x18\n\x16SetCustomHooksResponse\"H\n\x18\x42\x61\x63kupCustomHooksRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\"E\n\x15GetCustomHooksRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\")\n\x19\x42\x61\x63kupCustomHooksResponse\x12\x0c\n\x04\x64\x61ta\x18\x01 \x01(\x0c\"&\n\x16GetCustomHooksResponse\x12\x0c\n\x04\x64\x61ta\x18\x01 \x01(\x0c\"_\n!CreateRepositoryFromBundleRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\x12\x0c\n\x04\x64\x61ta\x18\x02 \x01(\x0c\"$\n\"CreateRepositoryFromBundleResponse\"B\n\x12\x46indLicenseRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\"\x8c\x01\n\x13\x46indLicenseResponse\x12\x1a\n\x12license_short_name\x18\x01 \x01(\t\x12\x14\n\x0clicense_name\x18\x02 \x01(\t\x12\x13\n\x0blicense_url\x18\x03 \x01(\t\x12\x14\n\x0clicense_path\x18\x04 \x01(\t\x12\x18\n\x10license_nickname\x18\x05 \x01(\t\"H\n\x18GetInfoAttributesRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\"/\n\x19GetInfoAttributesResponse\x12\x12\n\nattributes\x18\x01 \x01(\x0c\"H\n\x18\x43\x61lculateChecksumRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\"-\n\x19\x43\x61lculateChecksumResponse\x12\x10\n\x08\x63hecksum\x18\x01 \x01(\t\"B\n\x12GetSnapshotRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\"#\n\x13GetSnapshotResponse\x12\x0c\n\x04\x64\x61ta\x18\x01 \x01(\x0c\"\xa3\x01\n#CreateRepositoryFromSnapshotRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\x12\x10\n\x08http_url\x18\x02 \x01(\t\x12\x11\n\thttp_auth\x18\x03 \x01(\t\x12\x18\n\x10resolved_address\x18\x05 \x01(\tJ\x04\x08\x04\x10\x05R\thttp_host\"&\n$CreateRepositoryFromSnapshotResponse\"p\n\x14GetRawChangesRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\x12\x15\n\rfrom_revision\x18\x02 \x01(\t\x12\x13\n\x0bto_revision\x18\x03 \x01(\t\"\xbc\x03\n\x15GetRawChangesResponse\x12<\n\x0braw_changes\x18\x01 \x03(\x0b\x32\'.gitaly.GetRawChangesResponse.RawChange\x1a\xe4\x02\n\tRawChange\x12\x0f\n\x07\x62lob_id\x18\x01 \x01(\t\x12\x0c\n\x04size\x18\x02 \x01(\x03\x12\x44\n\toperation\x18\x05 \x01(\x0e\x32\x31.gitaly.GetRawChangesResponse.RawChange.Operation\x12\x10\n\x08old_mode\x18\x07 \x01(\x05\x12\x10\n\x08new_mode\x18\x08 \x01(\x05\x12\x16\n\x0enew_path_bytes\x18\t \x01(\x0c\x12\x16\n\x0eold_path_bytes\x18\n \x01(\x0c\"i\n\tOperation\x12\x0b\n\x07UNKNOWN\x10\x00\x12\t\n\x05\x41\x44\x44\x45\x44\x10\x01\x12\n\n\x06\x43OPIED\x10\x02\x12\x0b\n\x07\x44\x45LETED\x10\x03\x12\x0c\n\x08MODIFIED\x10\x04\x12\x0b\n\x07RENAMED\x10\x05\x12\x10\n\x0cTYPE_CHANGED\x10\x06J\x04\x08\x03\x10\x04J\x04\x08\x04\x10\x05J\x04\x08\x06\x10\x07R\x08new_pathR\x08old_pathR\rraw_operation\"\x93\x01\n\x18SearchFilesByNameRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\x12\r\n\x05query\x18\x02 \x01(\t\x12\x0b\n\x03ref\x18\x03 \x01(\x0c\x12\x0e\n\x06\x66ilter\x18\x04 \x01(\t\x12\r\n\x05limit\x18\x05 \x01(\r\x12\x0e\n\x06offset\x18\x06 \x01(\r\"*\n\x19SearchFilesByNameResponse\x12\r\n\x05\x66iles\x18\x01 \x03(\x0c\"\x7f\n\x1bSearchFilesByContentRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\x12\r\n\x05query\x18\x02 \x01(\t\x12\x0b\n\x03ref\x18\x03 \x01(\x0cJ\x04\x08\x04\x10\x05R\x10\x63hunked_response\"W\n\x1cSearchFilesByContentResponse\x12\x12\n\nmatch_data\x18\x02 \x01(\x0c\x12\x14\n\x0c\x65nd_of_match\x18\x03 \x01(\x08J\x04\x08\x01\x10\x02R\x07matches\"\x87\x01\n\x06Remote\x12\x0b\n\x03url\x18\x01 \x01(\t\x12!\n\x19http_authorization_header\x18\x03 \x01(\t\x12\x16\n\x0emirror_refmaps\x18\x04 \x03(\t\x12\x18\n\x10resolved_address\x18\x06 \x01(\tJ\x04\x08\x02\x10\x03J\x04\x08\x05\x10\x06R\x04nameR\thttp_host\"M\n\x1dGetObjectDirectorySizeRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\".\n\x1eGetObjectDirectorySizeResponse\x12\x0c\n\x04size\x18\x01 \x01(\x03\"G\n\x17RemoveRepositoryRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\"\x1a\n\x18RemoveRepositoryResponse\"n\n\x1aReplicateRepositoryRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\x12\"\n\x06source\x18\x02 \x01(\x0b\x32\x12.gitaly.Repository\"\x1d\n\x1bReplicateRepositoryResponse\"\xdb\x01\n\x19OptimizeRepositoryRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\x12<\n\x08strategy\x18\x02 \x01(\x0e\x32*.gitaly.OptimizeRepositoryRequest.Strategy\"R\n\x08Strategy\x12\x18\n\x14STRATEGY_UNSPECIFIED\x10\x00\x12\x18\n\x14STRATEGY_HEURISTICAL\x10\x01\x12\x12\n\x0eSTRATEGY_EAGER\x10\x02\"\x1c\n\x1aOptimizeRepositoryResponse\"N\n\x1ePruneUnreachableObjectsRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\"!\n\x1fPruneUnreachableObjectsResponse\"\x9e\x01\n\x17\x42\x61\x63kupRepositoryRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\x12-\n\x11vanity_repository\x18\x02 \x01(\x0b\x32\x12.gitaly.Repository\x12\x11\n\tbackup_id\x18\x03 \x01(\t\x12\x13\n\x0bincremental\x18\x04 \x01(\x08\"*\n\x18\x42\x61\x63kupRepositoryResponse\x1a\x0e\n\x0cSkippedError\"\xa1\x01\n\x18RestoreRepositoryRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\x12-\n\x11vanity_repository\x18\x02 \x01(\x0b\x32\x12.gitaly.Repository\x12\x11\n\tbackup_id\x18\x03 \x01(\t\x12\x15\n\ralways_create\x18\x04 \x01(\x08\"+\n\x19RestoreRepositoryResponse\x1a\x0e\n\x0cSkippedError\"}\n\x18GetFileAttributesRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\x12\x10\n\x08revision\x18\x02 \x01(\x0c\x12\x12\n\nattributes\x18\x03 \x03(\t\x12\r\n\x05paths\x18\x04 \x03(\t\"\xa6\x01\n\x19GetFileAttributesResponse\x12H\n\x0f\x61ttribute_infos\x18\x01 \x03(\x0b\x32/.gitaly.GetFileAttributesResponse.AttributeInfo\x1a?\n\rAttributeInfo\x12\x0c\n\x04path\x18\x01 \x01(\t\x12\x11\n\tattribute\x18\x02 \x01(\t\x12\r\n\x05value\x18\x03 \x01(\t\"A\n\x11\x46\x61stExportRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\"\"\n\x12\x46\x61stExportResponse\x12\x0c\n\x04\x64\x61ta\x18\x01 \x01(\x0c\x32\xf7\x1f\n\x11RepositoryService\x12]\n\x10RepositoryExists\x12\x1f.gitaly.RepositoryExistsRequest\x1a .gitaly.RepositoryExistsResponse\"\x06\xfa\x97(\x02\x08\x02\x12W\n\x0eRepositorySize\x12\x1d.gitaly.RepositorySizeRequest\x1a\x1e.gitaly.RepositorySizeResponse\"\x06\xfa\x97(\x02\x08\x02\x12W\n\x0eRepositoryInfo\x12\x1d.gitaly.RepositoryInfoRequest\x1a\x1e.gitaly.RepositoryInfoResponse\"\x06\xfa\x97(\x02\x08\x02\x12P\n\x0bObjectsSize\x12\x1a.gitaly.ObjectsSizeRequest\x1a\x1b.gitaly.ObjectsSizeResponse\"\x06\xfa\x97(\x02\x08\x02(\x01\x12Q\n\x0cObjectFormat\x12\x1b.gitaly.ObjectFormatRequest\x1a\x1c.gitaly.ObjectFormatResponse\"\x06\xfa\x97(\x02\x08\x02\x12\x66\n\x12\x41pplyGitattributes\x12!.gitaly.ApplyGitattributesRequest\x1a\".gitaly.ApplyGitattributesResponse\"\t\x88\x02\x01\xfa\x97(\x02\x08\x01\x12N\n\x0b\x46\x65tchRemote\x12\x1a.gitaly.FetchRemoteRequest\x1a\x1b.gitaly.FetchRemoteResponse\"\x06\xfa\x97(\x02\x08\x01\x12]\n\x10\x43reateRepository\x12\x1f.gitaly.CreateRepositoryRequest\x1a .gitaly.CreateRepositoryResponse\"\x06\xfa\x97(\x02\x08\x01\x12M\n\nGetArchive\x12\x19.gitaly.GetArchiveRequest\x1a\x1a.gitaly.GetArchiveResponse\"\x06\xfa\x97(\x02\x08\x02\x30\x01\x12]\n\x10HasLocalBranches\x12\x1f.gitaly.HasLocalBranchesRequest\x1a .gitaly.HasLocalBranchesResponse\"\x06\xfa\x97(\x02\x08\x02\x12`\n\x11\x46\x65tchSourceBranch\x12 .gitaly.FetchSourceBranchRequest\x1a!.gitaly.FetchSourceBranchResponse\"\x06\xfa\x97(\x02\x08\x01\x12\x39\n\x04\x46sck\x12\x13.gitaly.FsckRequest\x1a\x14.gitaly.FsckResponse\"\x06\xfa\x97(\x02\x08\x02\x12\x45\n\x08WriteRef\x12\x17.gitaly.WriteRefRequest\x1a\x18.gitaly.WriteRefResponse\"\x06\xfa\x97(\x02\x08\x01\x12T\n\rFindMergeBase\x12\x1c.gitaly.FindMergeBaseRequest\x1a\x1d.gitaly.FindMergeBaseResponse\"\x06\xfa\x97(\x02\x08\x02\x12K\n\nCreateFork\x12\x19.gitaly.CreateForkRequest\x1a\x1a.gitaly.CreateForkResponse\"\x06\xfa\x97(\x02\x08\x01\x12r\n\x17\x43reateRepositoryFromURL\x12&.gitaly.CreateRepositoryFromURLRequest\x1a\'.gitaly.CreateRepositoryFromURLResponse\"\x06\xfa\x97(\x02\x08\x01\x12S\n\x0c\x43reateBundle\x12\x1b.gitaly.CreateBundleRequest\x1a\x1c.gitaly.CreateBundleResponse\"\x06\xfa\x97(\x02\x08\x02\x30\x01\x12v\n\x17\x43reateBundleFromRefList\x12&.gitaly.CreateBundleFromRefListRequest\x1a\'.gitaly.CreateBundleFromRefListResponse\"\x06\xfa\x97(\x02\x08\x02(\x01\x30\x01\x12`\n\x11GenerateBundleURI\x12 .gitaly.GenerateBundleURIRequest\x1a!.gitaly.GenerateBundleURIResponse\"\x06\xfa\x97(\x02\x08\x02\x12P\n\x0b\x46\x65tchBundle\x12\x1a.gitaly.FetchBundleRequest\x1a\x1b.gitaly.FetchBundleResponse\"\x06\xfa\x97(\x02\x08\x01(\x01\x12}\n\x1a\x43reateRepositoryFromBundle\x12).gitaly.CreateRepositoryFromBundleRequest\x1a*.gitaly.CreateRepositoryFromBundleResponse\"\x06\xfa\x97(\x02\x08\x01(\x01\x12J\n\tGetConfig\x12\x18.gitaly.GetConfigRequest\x1a\x19.gitaly.GetConfigResponse\"\x06\xfa\x97(\x02\x08\x02\x30\x01\x12N\n\x0b\x46indLicense\x12\x1a.gitaly.FindLicenseRequest\x1a\x1b.gitaly.FindLicenseResponse\"\x06\xfa\x97(\x02\x08\x02\x12\x65\n\x11GetInfoAttributes\x12 .gitaly.GetInfoAttributesRequest\x1a!.gitaly.GetInfoAttributesResponse\"\t\x88\x02\x01\xfa\x97(\x02\x08\x02\x30\x01\x12`\n\x11\x43\x61lculateChecksum\x12 .gitaly.CalculateChecksumRequest\x1a!.gitaly.CalculateChecksumResponse\"\x06\xfa\x97(\x02\x08\x02\x12P\n\x0bGetSnapshot\x12\x1a.gitaly.GetSnapshotRequest\x1a\x1b.gitaly.GetSnapshotResponse\"\x06\xfa\x97(\x02\x08\x02\x30\x01\x12\x81\x01\n\x1c\x43reateRepositoryFromSnapshot\x12+.gitaly.CreateRepositoryFromSnapshotRequest\x1a,.gitaly.CreateRepositoryFromSnapshotResponse\"\x06\xfa\x97(\x02\x08\x01\x12V\n\rGetRawChanges\x12\x1c.gitaly.GetRawChangesRequest\x1a\x1d.gitaly.GetRawChangesResponse\"\x06\xfa\x97(\x02\x08\x02\x30\x01\x12k\n\x14SearchFilesByContent\x12#.gitaly.SearchFilesByContentRequest\x1a$.gitaly.SearchFilesByContentResponse\"\x06\xfa\x97(\x02\x08\x02\x30\x01\x12\x62\n\x11SearchFilesByName\x12 .gitaly.SearchFilesByNameRequest\x1a!.gitaly.SearchFilesByNameResponse\"\x06\xfa\x97(\x02\x08\x02\x30\x01\x12h\n\x12RestoreCustomHooks\x12!.gitaly.RestoreCustomHooksRequest\x1a\".gitaly.RestoreCustomHooksResponse\"\t\x88\x02\x01\xfa\x97(\x02\x08\x01(\x01\x12Y\n\x0eSetCustomHooks\x12\x1d.gitaly.SetCustomHooksRequest\x1a\x1e.gitaly.SetCustomHooksResponse\"\x06\xfa\x97(\x02\x08\x01(\x01\x12\x65\n\x11\x42\x61\x63kupCustomHooks\x12 .gitaly.BackupCustomHooksRequest\x1a!.gitaly.BackupCustomHooksResponse\"\t\x88\x02\x01\xfa\x97(\x02\x08\x02\x30\x01\x12Y\n\x0eGetCustomHooks\x12\x1d.gitaly.GetCustomHooksRequest\x1a\x1e.gitaly.GetCustomHooksResponse\"\x06\xfa\x97(\x02\x08\x02\x30\x01\x12o\n\x16GetObjectDirectorySize\x12%.gitaly.GetObjectDirectorySizeRequest\x1a&.gitaly.GetObjectDirectorySizeResponse\"\x06\xfa\x97(\x02\x08\x02\x12]\n\x10RemoveRepository\x12\x1f.gitaly.RemoveRepositoryRequest\x1a .gitaly.RemoveRepositoryResponse\"\x06\xfa\x97(\x02\x08\x01\x12\x66\n\x13ReplicateRepository\x12\".gitaly.ReplicateRepositoryRequest\x1a#.gitaly.ReplicateRepositoryResponse\"\x06\xfa\x97(\x02\x08\x01\x12\x63\n\x12OptimizeRepository\x12!.gitaly.OptimizeRepositoryRequest\x1a\".gitaly.OptimizeRepositoryResponse\"\x06\xfa\x97(\x02\x08\x03\x12r\n\x17PruneUnreachableObjects\x12&.gitaly.PruneUnreachableObjectsRequest\x1a\'.gitaly.PruneUnreachableObjectsResponse\"\x06\xfa\x97(\x02\x08\x03\x12]\n\x10\x42\x61\x63kupRepository\x12\x1f.gitaly.BackupRepositoryRequest\x1a .gitaly.BackupRepositoryResponse\"\x06\xfa\x97(\x02\x08\x02\x12`\n\x11RestoreRepository\x12 .gitaly.RestoreRepositoryRequest\x1a!.gitaly.RestoreRepositoryResponse\"\x06\xfa\x97(\x02\x08\x01\x12`\n\x11GetFileAttributes\x12 .gitaly.GetFileAttributesRequest\x1a!.gitaly.GetFileAttributesResponse\"\x06\xfa\x97(\x02\x08\x02\x12M\n\nFastExport\x12\x19.gitaly.FastExportRequest\x1a\x1a.gitaly.FastExportResponse\"\x06\xfa\x97(\x02\x08\x02\x30\x01\x42\x34Z2gitlab.com/gitlab-org/gitaly/v16/proto/go/gitalypbb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'repository_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'Z2gitlab.com/gitlab-org/gitaly/v16/proto/go/gitalypb'
  _REPOSITORYEXISTSREQUEST.fields_by_name['repository']._options = None
  _REPOSITORYEXISTSREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _REPOSITORYSIZEREQUEST.fields_by_name['repository']._options = None
  _REPOSITORYSIZEREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _REPOSITORYINFOREQUEST.fields_by_name['repository']._options = None
  _REPOSITORYINFOREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _OBJECTSSIZEREQUEST.fields_by_name['repository']._options = None
  _OBJECTSSIZEREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _OBJECTFORMATREQUEST.fields_by_name['repository']._options = None
  _OBJECTFORMATREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _APPLYGITATTRIBUTESREQUEST.fields_by_name['repository']._options = None
  _APPLYGITATTRIBUTESREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _FETCHBUNDLEREQUEST.fields_by_name['repository']._options = None
  _FETCHBUNDLEREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _FETCHREMOTEREQUEST.fields_by_name['repository']._options = None
  _FETCHREMOTEREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _CREATEREPOSITORYREQUEST.fields_by_name['repository']._options = None
  _CREATEREPOSITORYREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _GETARCHIVEREQUEST.fields_by_name['repository']._options = None
  _GETARCHIVEREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _HASLOCALBRANCHESREQUEST.fields_by_name['repository']._options = None
  _HASLOCALBRANCHESREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _FETCHSOURCEBRANCHREQUEST.fields_by_name['repository']._options = None
  _FETCHSOURCEBRANCHREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _FSCKREQUEST.fields_by_name['repository']._options = None
  _FSCKREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _WRITEREFREQUEST.fields_by_name['repository']._options = None
  _WRITEREFREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _FINDMERGEBASEREQUEST.fields_by_name['repository']._options = None
  _FINDMERGEBASEREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _CREATEFORKREQUEST.fields_by_name['repository']._options = None
  _CREATEFORKREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _CREATEREPOSITORYFROMURLREQUEST.fields_by_name['repository']._options = None
  _CREATEREPOSITORYFROMURLREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _CREATEBUNDLEREQUEST.fields_by_name['repository']._options = None
  _CREATEBUNDLEREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _CREATEBUNDLEFROMREFLISTREQUEST.fields_by_name['repository']._options = None
  _CREATEBUNDLEFROMREFLISTREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _GENERATEBUNDLEURIREQUEST.fields_by_name['repository']._options = None
  _GENERATEBUNDLEURIREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _GETCONFIGREQUEST.fields_by_name['repository']._options = None
  _GETCONFIGREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _RESTORECUSTOMHOOKSREQUEST.fields_by_name['repository']._options = None
  _RESTORECUSTOMHOOKSREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _SETCUSTOMHOOKSREQUEST.fields_by_name['repository']._options = None
  _SETCUSTOMHOOKSREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _BACKUPCUSTOMHOOKSREQUEST.fields_by_name['repository']._options = None
  _BACKUPCUSTOMHOOKSREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _GETCUSTOMHOOKSREQUEST.fields_by_name['repository']._options = None
  _GETCUSTOMHOOKSREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _CREATEREPOSITORYFROMBUNDLEREQUEST.fields_by_name['repository']._options = None
  _CREATEREPOSITORYFROMBUNDLEREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _FINDLICENSEREQUEST.fields_by_name['repository']._options = None
  _FINDLICENSEREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _GETINFOATTRIBUTESREQUEST.fields_by_name['repository']._options = None
  _GETINFOATTRIBUTESREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _CALCULATECHECKSUMREQUEST.fields_by_name['repository']._options = None
  _CALCULATECHECKSUMREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _GETSNAPSHOTREQUEST.fields_by_name['repository']._options = None
  _GETSNAPSHOTREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _CREATEREPOSITORYFROMSNAPSHOTREQUEST.fields_by_name['repository']._options = None
  _CREATEREPOSITORYFROMSNAPSHOTREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _GETRAWCHANGESREQUEST.fields_by_name['repository']._options = None
  _GETRAWCHANGESREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _SEARCHFILESBYNAMEREQUEST.fields_by_name['repository']._options = None
  _SEARCHFILESBYNAMEREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _SEARCHFILESBYCONTENTREQUEST.fields_by_name['repository']._options = None
  _SEARCHFILESBYCONTENTREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _GETOBJECTDIRECTORYSIZEREQUEST.fields_by_name['repository']._options = None
  _GETOBJECTDIRECTORYSIZEREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _REMOVEREPOSITORYREQUEST.fields_by_name['repository']._options = None
  _REMOVEREPOSITORYREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _REPLICATEREPOSITORYREQUEST.fields_by_name['repository']._options = None
  _REPLICATEREPOSITORYREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _OPTIMIZEREPOSITORYREQUEST.fields_by_name['repository']._options = None
  _OPTIMIZEREPOSITORYREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _PRUNEUNREACHABLEOBJECTSREQUEST.fields_by_name['repository']._options = None
  _PRUNEUNREACHABLEOBJECTSREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _BACKUPREPOSITORYREQUEST.fields_by_name['repository']._options = None
  _BACKUPREPOSITORYREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _RESTOREREPOSITORYREQUEST.fields_by_name['repository']._options = None
  _RESTOREREPOSITORYREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _GETFILEATTRIBUTESREQUEST.fields_by_name['repository']._options = None
  _GETFILEATTRIBUTESREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _FASTEXPORTREQUEST.fields_by_name['repository']._options = None
  _FASTEXPORTREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _REPOSITORYSERVICE.methods_by_name['RepositoryExists']._options = None
  _REPOSITORYSERVICE.methods_by_name['RepositoryExists']._serialized_options = b'\372\227(\002\010\002'
  _REPOSITORYSERVICE.methods_by_name['RepositorySize']._options = None
  _REPOSITORYSERVICE.methods_by_name['RepositorySize']._serialized_options = b'\372\227(\002\010\002'
  _REPOSITORYSERVICE.methods_by_name['RepositoryInfo']._options = None
  _REPOSITORYSERVICE.methods_by_name['RepositoryInfo']._serialized_options = b'\372\227(\002\010\002'
  _REPOSITORYSERVICE.methods_by_name['ObjectsSize']._options = None
  _REPOSITORYSERVICE.methods_by_name['ObjectsSize']._serialized_options = b'\372\227(\002\010\002'
  _REPOSITORYSERVICE.methods_by_name['ObjectFormat']._options = None
  _REPOSITORYSERVICE.methods_by_name['ObjectFormat']._serialized_options = b'\372\227(\002\010\002'
  _REPOSITORYSERVICE.methods_by_name['ApplyGitattributes']._options = None
  _REPOSITORYSERVICE.methods_by_name['ApplyGitattributes']._serialized_options = b'\210\002\001\372\227(\002\010\001'
  _REPOSITORYSERVICE.methods_by_name['FetchRemote']._options = None
  _REPOSITORYSERVICE.methods_by_name['FetchRemote']._serialized_options = b'\372\227(\002\010\001'
  _REPOSITORYSERVICE.methods_by_name['CreateRepository']._options = None
  _REPOSITORYSERVICE.methods_by_name['CreateRepository']._serialized_options = b'\372\227(\002\010\001'
  _REPOSITORYSERVICE.methods_by_name['GetArchive']._options = None
  _REPOSITORYSERVICE.methods_by_name['GetArchive']._serialized_options = b'\372\227(\002\010\002'
  _REPOSITORYSERVICE.methods_by_name['HasLocalBranches']._options = None
  _REPOSITORYSERVICE.methods_by_name['HasLocalBranches']._serialized_options = b'\372\227(\002\010\002'
  _REPOSITORYSERVICE.methods_by_name['FetchSourceBranch']._options = None
  _REPOSITORYSERVICE.methods_by_name['FetchSourceBranch']._serialized_options = b'\372\227(\002\010\001'
  _REPOSITORYSERVICE.methods_by_name['Fsck']._options = None
  _REPOSITORYSERVICE.methods_by_name['Fsck']._serialized_options = b'\372\227(\002\010\002'
  _REPOSITORYSERVICE.methods_by_name['WriteRef']._options = None
  _REPOSITORYSERVICE.methods_by_name['WriteRef']._serialized_options = b'\372\227(\002\010\001'
  _REPOSITORYSERVICE.methods_by_name['FindMergeBase']._options = None
  _REPOSITORYSERVICE.methods_by_name['FindMergeBase']._serialized_options = b'\372\227(\002\010\002'
  _REPOSITORYSERVICE.methods_by_name['CreateFork']._options = None
  _REPOSITORYSERVICE.methods_by_name['CreateFork']._serialized_options = b'\372\227(\002\010\001'
  _REPOSITORYSERVICE.methods_by_name['CreateRepositoryFromURL']._options = None
  _REPOSITORYSERVICE.methods_by_name['CreateRepositoryFromURL']._serialized_options = b'\372\227(\002\010\001'
  _REPOSITORYSERVICE.methods_by_name['CreateBundle']._options = None
  _REPOSITORYSERVICE.methods_by_name['CreateBundle']._serialized_options = b'\372\227(\002\010\002'
  _REPOSITORYSERVICE.methods_by_name['CreateBundleFromRefList']._options = None
  _REPOSITORYSERVICE.methods_by_name['CreateBundleFromRefList']._serialized_options = b'\372\227(\002\010\002'
  _REPOSITORYSERVICE.methods_by_name['GenerateBundleURI']._options = None
  _REPOSITORYSERVICE.methods_by_name['GenerateBundleURI']._serialized_options = b'\372\227(\002\010\002'
  _REPOSITORYSERVICE.methods_by_name['FetchBundle']._options = None
  _REPOSITORYSERVICE.methods_by_name['FetchBundle']._serialized_options = b'\372\227(\002\010\001'
  _REPOSITORYSERVICE.methods_by_name['CreateRepositoryFromBundle']._options = None
  _REPOSITORYSERVICE.methods_by_name['CreateRepositoryFromBundle']._serialized_options = b'\372\227(\002\010\001'
  _REPOSITORYSERVICE.methods_by_name['GetConfig']._options = None
  _REPOSITORYSERVICE.methods_by_name['GetConfig']._serialized_options = b'\372\227(\002\010\002'
  _REPOSITORYSERVICE.methods_by_name['FindLicense']._options = None
  _REPOSITORYSERVICE.methods_by_name['FindLicense']._serialized_options = b'\372\227(\002\010\002'
  _REPOSITORYSERVICE.methods_by_name['GetInfoAttributes']._options = None
  _REPOSITORYSERVICE.methods_by_name['GetInfoAttributes']._serialized_options = b'\210\002\001\372\227(\002\010\002'
  _REPOSITORYSERVICE.methods_by_name['CalculateChecksum']._options = None
  _REPOSITORYSERVICE.methods_by_name['CalculateChecksum']._serialized_options = b'\372\227(\002\010\002'
  _REPOSITORYSERVICE.methods_by_name['GetSnapshot']._options = None
  _REPOSITORYSERVICE.methods_by_name['GetSnapshot']._serialized_options = b'\372\227(\002\010\002'
  _REPOSITORYSERVICE.methods_by_name['CreateRepositoryFromSnapshot']._options = None
  _REPOSITORYSERVICE.methods_by_name['CreateRepositoryFromSnapshot']._serialized_options = b'\372\227(\002\010\001'
  _REPOSITORYSERVICE.methods_by_name['GetRawChanges']._options = None
  _REPOSITORYSERVICE.methods_by_name['GetRawChanges']._serialized_options = b'\372\227(\002\010\002'
  _REPOSITORYSERVICE.methods_by_name['SearchFilesByContent']._options = None
  _REPOSITORYSERVICE.methods_by_name['SearchFilesByContent']._serialized_options = b'\372\227(\002\010\002'
  _REPOSITORYSERVICE.methods_by_name['SearchFilesByName']._options = None
  _REPOSITORYSERVICE.methods_by_name['SearchFilesByName']._serialized_options = b'\372\227(\002\010\002'
  _REPOSITORYSERVICE.methods_by_name['RestoreCustomHooks']._options = None
  _REPOSITORYSERVICE.methods_by_name['RestoreCustomHooks']._serialized_options = b'\210\002\001\372\227(\002\010\001'
  _REPOSITORYSERVICE.methods_by_name['SetCustomHooks']._options = None
  _REPOSITORYSERVICE.methods_by_name['SetCustomHooks']._serialized_options = b'\372\227(\002\010\001'
  _REPOSITORYSERVICE.methods_by_name['BackupCustomHooks']._options = None
  _REPOSITORYSERVICE.methods_by_name['BackupCustomHooks']._serialized_options = b'\210\002\001\372\227(\002\010\002'
  _REPOSITORYSERVICE.methods_by_name['GetCustomHooks']._options = None
  _REPOSITORYSERVICE.methods_by_name['GetCustomHooks']._serialized_options = b'\372\227(\002\010\002'
  _REPOSITORYSERVICE.methods_by_name['GetObjectDirectorySize']._options = None
  _REPOSITORYSERVICE.methods_by_name['GetObjectDirectorySize']._serialized_options = b'\372\227(\002\010\002'
  _REPOSITORYSERVICE.methods_by_name['RemoveRepository']._options = None
  _REPOSITORYSERVICE.methods_by_name['RemoveRepository']._serialized_options = b'\372\227(\002\010\001'
  _REPOSITORYSERVICE.methods_by_name['ReplicateRepository']._options = None
  _REPOSITORYSERVICE.methods_by_name['ReplicateRepository']._serialized_options = b'\372\227(\002\010\001'
  _REPOSITORYSERVICE.methods_by_name['OptimizeRepository']._options = None
  _REPOSITORYSERVICE.methods_by_name['OptimizeRepository']._serialized_options = b'\372\227(\002\010\003'
  _REPOSITORYSERVICE.methods_by_name['PruneUnreachableObjects']._options = None
  _REPOSITORYSERVICE.methods_by_name['PruneUnreachableObjects']._serialized_options = b'\372\227(\002\010\003'
  _REPOSITORYSERVICE.methods_by_name['BackupRepository']._options = None
  _REPOSITORYSERVICE.methods_by_name['BackupRepository']._serialized_options = b'\372\227(\002\010\002'
  _REPOSITORYSERVICE.methods_by_name['RestoreRepository']._options = None
  _REPOSITORYSERVICE.methods_by_name['RestoreRepository']._serialized_options = b'\372\227(\002\010\001'
  _REPOSITORYSERVICE.methods_by_name['GetFileAttributes']._options = None
  _REPOSITORYSERVICE.methods_by_name['GetFileAttributes']._serialized_options = b'\372\227(\002\010\002'
  _REPOSITORYSERVICE.methods_by_name['FastExport']._options = None
  _REPOSITORYSERVICE.methods_by_name['FastExport']._serialized_options = b'\372\227(\002\010\002'
  _globals['_REPOSITORYEXISTSREQUEST']._serialized_start=54
  _globals['_REPOSITORYEXISTSREQUEST']._serialized_end=125
  _globals['_REPOSITORYEXISTSRESPONSE']._serialized_start=127
  _globals['_REPOSITORYEXISTSRESPONSE']._serialized_end=169
  _globals['_REPOSITORYSIZEREQUEST']._serialized_start=171
  _globals['_REPOSITORYSIZEREQUEST']._serialized_end=240
  _globals['_REPOSITORYSIZERESPONSE']._serialized_start=242
  _globals['_REPOSITORYSIZERESPONSE']._serialized_end=280
  _globals['_REPOSITORYINFOREQUEST']._serialized_start=282
  _globals['_REPOSITORYINFOREQUEST']._serialized_end=351
  _globals['_REPOSITORYINFORESPONSE']._serialized_start=354
  _globals['_REPOSITORYINFORESPONSE']._serialized_end=669
  _globals['_REPOSITORYINFORESPONSE_REFERENCESINFO']._serialized_start=522
  _globals['_REPOSITORYINFORESPONSE_REFERENCESINFO']._serialized_end=580
  _globals['_REPOSITORYINFORESPONSE_OBJECTSINFO']._serialized_start=582
  _globals['_REPOSITORYINFORESPONSE_OBJECTSINFO']._serialized_end=669
  _globals['_OBJECTSSIZEREQUEST']._serialized_start=671
  _globals['_OBJECTSSIZEREQUEST']._serialized_end=756
  _globals['_OBJECTSSIZERESPONSE']._serialized_start=758
  _globals['_OBJECTSSIZERESPONSE']._serialized_end=793
  _globals['_OBJECTFORMATREQUEST']._serialized_start=795
  _globals['_OBJECTFORMATREQUEST']._serialized_end=862
  _globals['_OBJECTFORMATRESPONSE']._serialized_start=864
  _globals['_OBJECTFORMATRESPONSE']._serialized_end=924
  _globals['_APPLYGITATTRIBUTESREQUEST']._serialized_start=926
  _globals['_APPLYGITATTRIBUTESREQUEST']._serialized_end=1017
  _globals['_APPLYGITATTRIBUTESRESPONSE']._serialized_start=1019
  _globals['_APPLYGITATTRIBUTESRESPONSE']._serialized_end=1047
  _globals['_FETCHBUNDLEREQUEST']._serialized_start=1049
  _globals['_FETCHBUNDLEREQUEST']._serialized_end=1150
  _globals['_FETCHBUNDLERESPONSE']._serialized_start=1152
  _globals['_FETCHBUNDLERESPONSE']._serialized_end=1173
  _globals['_FETCHREMOTEREQUEST']._serialized_start=1176
  _globals['_FETCHREMOTEREQUEST']._serialized_end=1434
  _globals['_FETCHREMOTERESPONSE']._serialized_start=1436
  _globals['_FETCHREMOTERESPONSE']._serialized_end=1479
  _globals['_CREATEREPOSITORYREQUEST']._serialized_start=1482
  _globals['_CREATEREPOSITORYREQUEST']._serialized_end=1622
  _globals['_CREATEREPOSITORYRESPONSE']._serialized_start=1624
  _globals['_CREATEREPOSITORYRESPONSE']._serialized_end=1650
  _globals['_GETARCHIVEREQUEST']._serialized_start=1653
  _globals['_GETARCHIVEREQUEST']._serialized_end=1934
  _globals['_GETARCHIVEREQUEST_FORMAT']._serialized_start=1883
  _globals['_GETARCHIVEREQUEST_FORMAT']._serialized_end=1934
  _globals['_GETARCHIVERESPONSE']._serialized_start=1936
  _globals['_GETARCHIVERESPONSE']._serialized_end=1970
  _globals['_HASLOCALBRANCHESREQUEST']._serialized_start=1972
  _globals['_HASLOCALBRANCHESREQUEST']._serialized_end=2043
  _globals['_HASLOCALBRANCHESRESPONSE']._serialized_start=2045
  _globals['_HASLOCALBRANCHESRESPONSE']._serialized_end=2086
  _globals['_FETCHSOURCEBRANCHREQUEST']._serialized_start=2089
  _globals['_FETCHSOURCEBRANCHREQUEST']._serialized_end=2284
  _globals['_FETCHSOURCEBRANCHRESPONSE']._serialized_start=2286
  _globals['_FETCHSOURCEBRANCHRESPONSE']._serialized_end=2329
  _globals['_FSCKREQUEST']._serialized_start=2331
  _globals['_FSCKREQUEST']._serialized_end=2390
  _globals['_FSCKRESPONSE']._serialized_start=2392
  _globals['_FSCKRESPONSE']._serialized_end=2421
  _globals['_WRITEREFREQUEST']._serialized_start=2424
  _globals['_WRITEREFREQUEST']._serialized_end=2559
  _globals['_WRITEREFRESPONSE']._serialized_start=2561
  _globals['_WRITEREFRESPONSE']._serialized_end=2585
  _globals['_FINDMERGEBASEREQUEST']._serialized_start=2587
  _globals['_FINDMERGEBASEREQUEST']._serialized_end=2674
  _globals['_FINDMERGEBASERESPONSE']._serialized_start=2676
  _globals['_FINDMERGEBASERESPONSE']._serialized_end=2713
  _globals['_CREATEFORKREQUEST']._serialized_start=2716
  _globals['_CREATEFORKREQUEST']._serialized_end=2846
  _globals['_CREATEFORKRESPONSE']._serialized_start=2848
  _globals['_CREATEFORKRESPONSE']._serialized_end=2868
  _globals['_CREATEREPOSITORYFROMURLREQUEST']._serialized_start=2871
  _globals['_CREATEREPOSITORYFROMURLREQUEST']._serialized_end=3056
  _globals['_CREATEREPOSITORYFROMURLRESPONSE']._serialized_start=3058
  _globals['_CREATEREPOSITORYFROMURLRESPONSE']._serialized_end=3091
  _globals['_CREATEBUNDLEREQUEST']._serialized_start=3093
  _globals['_CREATEBUNDLEREQUEST']._serialized_end=3160
  _globals['_CREATEBUNDLERESPONSE']._serialized_start=3162
  _globals['_CREATEBUNDLERESPONSE']._serialized_end=3198
  _globals['_CREATEBUNDLEFROMREFLISTREQUEST']._serialized_start=3200
  _globals['_CREATEBUNDLEFROMREFLISTREQUEST']._serialized_end=3296
  _globals['_CREATEBUNDLEFROMREFLISTRESPONSE']._serialized_start=3298
  _globals['_CREATEBUNDLEFROMREFLISTRESPONSE']._serialized_end=3345
  _globals['_GENERATEBUNDLEURIREQUEST']._serialized_start=3347
  _globals['_GENERATEBUNDLEURIREQUEST']._serialized_end=3419
  _globals['_GENERATEBUNDLEURIRESPONSE']._serialized_start=3421
  _globals['_GENERATEBUNDLEURIRESPONSE']._serialized_end=3448
  _globals['_GETCONFIGREQUEST']._serialized_start=3450
  _globals['_GETCONFIGREQUEST']._serialized_end=3514
  _globals['_GETCONFIGRESPONSE']._serialized_start=3516
  _globals['_GETCONFIGRESPONSE']._serialized_end=3549
  _globals['_RESTORECUSTOMHOOKSREQUEST']._serialized_start=3551
  _globals['_RESTORECUSTOMHOOKSREQUEST']._serialized_end=3638
  _globals['_SETCUSTOMHOOKSREQUEST']._serialized_start=3640
  _globals['_SETCUSTOMHOOKSREQUEST']._serialized_end=3723
  _globals['_RESTORECUSTOMHOOKSRESPONSE']._serialized_start=3725
  _globals['_RESTORECUSTOMHOOKSRESPONSE']._serialized_end=3753
  _globals['_SETCUSTOMHOOKSRESPONSE']._serialized_start=3755
  _globals['_SETCUSTOMHOOKSRESPONSE']._serialized_end=3779
  _globals['_BACKUPCUSTOMHOOKSREQUEST']._serialized_start=3781
  _globals['_BACKUPCUSTOMHOOKSREQUEST']._serialized_end=3853
  _globals['_GETCUSTOMHOOKSREQUEST']._serialized_start=3855
  _globals['_GETCUSTOMHOOKSREQUEST']._serialized_end=3924
  _globals['_BACKUPCUSTOMHOOKSRESPONSE']._serialized_start=3926
  _globals['_BACKUPCUSTOMHOOKSRESPONSE']._serialized_end=3967
  _globals['_GETCUSTOMHOOKSRESPONSE']._serialized_start=3969
  _globals['_GETCUSTOMHOOKSRESPONSE']._serialized_end=4007
  _globals['_CREATEREPOSITORYFROMBUNDLEREQUEST']._serialized_start=4009
  _globals['_CREATEREPOSITORYFROMBUNDLEREQUEST']._serialized_end=4104
  _globals['_CREATEREPOSITORYFROMBUNDLERESPONSE']._serialized_start=4106
  _globals['_CREATEREPOSITORYFROMBUNDLERESPONSE']._serialized_end=4142
  _globals['_FINDLICENSEREQUEST']._serialized_start=4144
  _globals['_FINDLICENSEREQUEST']._serialized_end=4210
  _globals['_FINDLICENSERESPONSE']._serialized_start=4213
  _globals['_FINDLICENSERESPONSE']._serialized_end=4353
  _globals['_GETINFOATTRIBUTESREQUEST']._serialized_start=4355
  _globals['_GETINFOATTRIBUTESREQUEST']._serialized_end=4427
  _globals['_GETINFOATTRIBUTESRESPONSE']._serialized_start=4429
  _globals['_GETINFOATTRIBUTESRESPONSE']._serialized_end=4476
  _globals['_CALCULATECHECKSUMREQUEST']._serialized_start=4478
  _globals['_CALCULATECHECKSUMREQUEST']._serialized_end=4550
  _globals['_CALCULATECHECKSUMRESPONSE']._serialized_start=4552
  _globals['_CALCULATECHECKSUMRESPONSE']._serialized_end=4597
  _globals['_GETSNAPSHOTREQUEST']._serialized_start=4599
  _globals['_GETSNAPSHOTREQUEST']._serialized_end=4665
  _globals['_GETSNAPSHOTRESPONSE']._serialized_start=4667
  _globals['_GETSNAPSHOTRESPONSE']._serialized_end=4702
  _globals['_CREATEREPOSITORYFROMSNAPSHOTREQUEST']._serialized_start=4705
  _globals['_CREATEREPOSITORYFROMSNAPSHOTREQUEST']._serialized_end=4868
  _globals['_CREATEREPOSITORYFROMSNAPSHOTRESPONSE']._serialized_start=4870
  _globals['_CREATEREPOSITORYFROMSNAPSHOTRESPONSE']._serialized_end=4908
  _globals['_GETRAWCHANGESREQUEST']._serialized_start=4910
  _globals['_GETRAWCHANGESREQUEST']._serialized_end=5022
  _globals['_GETRAWCHANGESRESPONSE']._serialized_start=5025
  _globals['_GETRAWCHANGESRESPONSE']._serialized_end=5469
  _globals['_GETRAWCHANGESRESPONSE_RAWCHANGE']._serialized_start=5113
  _globals['_GETRAWCHANGESRESPONSE_RAWCHANGE']._serialized_end=5469
  _globals['_GETRAWCHANGESRESPONSE_RAWCHANGE_OPERATION']._serialized_start=5311
  _globals['_GETRAWCHANGESRESPONSE_RAWCHANGE_OPERATION']._serialized_end=5416
  _globals['_SEARCHFILESBYNAMEREQUEST']._serialized_start=5472
  _globals['_SEARCHFILESBYNAMEREQUEST']._serialized_end=5619
  _globals['_SEARCHFILESBYNAMERESPONSE']._serialized_start=5621
  _globals['_SEARCHFILESBYNAMERESPONSE']._serialized_end=5663
  _globals['_SEARCHFILESBYCONTENTREQUEST']._serialized_start=5665
  _globals['_SEARCHFILESBYCONTENTREQUEST']._serialized_end=5792
  _globals['_SEARCHFILESBYCONTENTRESPONSE']._serialized_start=5794
  _globals['_SEARCHFILESBYCONTENTRESPONSE']._serialized_end=5881
  _globals['_REMOTE']._serialized_start=5884
  _globals['_REMOTE']._serialized_end=6019
  _globals['_GETOBJECTDIRECTORYSIZEREQUEST']._serialized_start=6021
  _globals['_GETOBJECTDIRECTORYSIZEREQUEST']._serialized_end=6098
  _globals['_GETOBJECTDIRECTORYSIZERESPONSE']._serialized_start=6100
  _globals['_GETOBJECTDIRECTORYSIZERESPONSE']._serialized_end=6146
  _globals['_REMOVEREPOSITORYREQUEST']._serialized_start=6148
  _globals['_REMOVEREPOSITORYREQUEST']._serialized_end=6219
  _globals['_REMOVEREPOSITORYRESPONSE']._serialized_start=6221
  _globals['_REMOVEREPOSITORYRESPONSE']._serialized_end=6247
  _globals['_REPLICATEREPOSITORYREQUEST']._serialized_start=6249
  _globals['_REPLICATEREPOSITORYREQUEST']._serialized_end=6359
  _globals['_REPLICATEREPOSITORYRESPONSE']._serialized_start=6361
  _globals['_REPLICATEREPOSITORYRESPONSE']._serialized_end=6390
  _globals['_OPTIMIZEREPOSITORYREQUEST']._serialized_start=6393
  _globals['_OPTIMIZEREPOSITORYREQUEST']._serialized_end=6612
  _globals['_OPTIMIZEREPOSITORYREQUEST_STRATEGY']._serialized_start=6530
  _globals['_OPTIMIZEREPOSITORYREQUEST_STRATEGY']._serialized_end=6612
  _globals['_OPTIMIZEREPOSITORYRESPONSE']._serialized_start=6614
  _globals['_OPTIMIZEREPOSITORYRESPONSE']._serialized_end=6642
  _globals['_PRUNEUNREACHABLEOBJECTSREQUEST']._serialized_start=6644
  _globals['_PRUNEUNREACHABLEOBJECTSREQUEST']._serialized_end=6722
  _globals['_PRUNEUNREACHABLEOBJECTSRESPONSE']._serialized_start=6724
  _globals['_PRUNEUNREACHABLEOBJECTSRESPONSE']._serialized_end=6757
  _globals['_BACKUPREPOSITORYREQUEST']._serialized_start=6760
  _globals['_BACKUPREPOSITORYREQUEST']._serialized_end=6918
  _globals['_BACKUPREPOSITORYRESPONSE']._serialized_start=6920
  _globals['_BACKUPREPOSITORYRESPONSE']._serialized_end=6962
  _globals['_BACKUPREPOSITORYRESPONSE_SKIPPEDERROR']._serialized_start=6948
  _globals['_BACKUPREPOSITORYRESPONSE_SKIPPEDERROR']._serialized_end=6962
  _globals['_RESTOREREPOSITORYREQUEST']._serialized_start=6965
  _globals['_RESTOREREPOSITORYREQUEST']._serialized_end=7126
  _globals['_RESTOREREPOSITORYRESPONSE']._serialized_start=7128
  _globals['_RESTOREREPOSITORYRESPONSE']._serialized_end=7171
  _globals['_RESTOREREPOSITORYRESPONSE_SKIPPEDERROR']._serialized_start=6948
  _globals['_RESTOREREPOSITORYRESPONSE_SKIPPEDERROR']._serialized_end=6962
  _globals['_GETFILEATTRIBUTESREQUEST']._serialized_start=7173
  _globals['_GETFILEATTRIBUTESREQUEST']._serialized_end=7298
  _globals['_GETFILEATTRIBUTESRESPONSE']._serialized_start=7301
  _globals['_GETFILEATTRIBUTESRESPONSE']._serialized_end=7467
  _globals['_GETFILEATTRIBUTESRESPONSE_ATTRIBUTEINFO']._serialized_start=7404
  _globals['_GETFILEATTRIBUTESRESPONSE_ATTRIBUTEINFO']._serialized_end=7467
  _globals['_FASTEXPORTREQUEST']._serialized_start=7469
  _globals['_FASTEXPORTREQUEST']._serialized_end=7534
  _globals['_FASTEXPORTRESPONSE']._serialized_start=7536
  _globals['_FASTEXPORTRESPONSE']._serialized_end=7570
  _globals['_REPOSITORYSERVICE']._serialized_start=7573
  _globals['_REPOSITORYSERVICE']._serialized_end=11660
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: errors.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import duration_pb2 as google_dot_protobuf_dot_duration__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x0c\x65rrors.proto\x12\x06gitaly\x1a\x1egoogle/protobuf/duration.proto\"]\n\x10\x41\x63\x63\x65ssCheckError\x12\x15\n\rerror_message\x18\x01 \x01(\t\x12\x10\n\x08protocol\x18\x02 \x01(\t\x12\x0f\n\x07user_id\x18\x03 \x01(\t\x12\x0f\n\x07\x63hanges\x18\x04 \x01(\x0c\"\xae\x02\n\nIndexError\x12\x0c\n\x04path\x18\x01 \x01(\x0c\x12\x30\n\nerror_type\x18\x02 \x01(\x0e\x32\x1c.gitaly.IndexError.ErrorType\"\xdf\x01\n\tErrorType\x12\x1a\n\x16\x45RROR_TYPE_UNSPECIFIED\x10\x00\x12\x19\n\x15\x45RROR_TYPE_EMPTY_PATH\x10\x01\x12\x1b\n\x17\x45RROR_TYPE_INVALID_PATH\x10\x02\x12\x1f\n\x1b\x45RROR_TYPE_DIRECTORY_EXISTS\x10\x03\x12\"\n\x1e\x45RROR_TYPE_DIRECTORY_TRAVERSAL\x10\x04\x12\x1a\n\x16\x45RROR_TYPE_FILE_EXISTS\x10\x05\x12\x1d\n\x19\x45RROR_TYPE_FILE_NOT_FOUND\x10\x06\"%\n\x15InvalidRefFormatError\x12\x0c\n\x04refs\x18\x02 \x03(\x0c\"C\n\x10NotAncestorError\x12\x17\n\x0fparent_revision\x18\x01 \x01(\x0c\x12\x16\n\x0e\x63hild_revision\x18\x02 \x01(\x0c\"\x1c\n\x1a\x43hangesAlreadyAppliedError\"O\n\x12MergeConflictError\x12\x19\n\x11\x63onflicting_files\x18\x01 \x03(\x0c\x12\x1e\n\x16\x63onflicting_commit_ids\x18\x02 \x03(\t\"%\n\x15ReferencesLockedError\x12\x0c\n\x04refs\x18\x01 \x03(\x0c\";\n\x14ReferenceExistsError\x12\x16\n\x0ereference_name\x18\x01 \x01(\x0c\x12\x0b\n\x03oid\x18\x02 \x01(\t\"0\n\x16ReferenceNotFoundError\x12\x16\n\x0ereference_name\x18\x01 \x01(\x0c\"k\n\x1bReferenceStateMismatchError\x12\x16\n\x0ereference_name\x18\x01 \x01(\x0c\x12\x1a\n\x12\x65xpected_object_id\x18\x02 \x01(\x0c\x12\x18\n\x10\x61\x63tual_object_id\x18\x03 \x01(\x0c\"P\n\x14ReferenceUpdateError\x12\x16\n\x0ereference_name\x18\x01 \x01(\x0c\x12\x0f\n\x07old_oid\x18\x02 \x01(\t\x12\x0f\n\x07new_oid\x18\x03 \x01(\t\"(\n\x14ResolveRevisionError\x12\x10\n\x08revision\x18\x01 \x01(\x0c\"S\n\nLimitError\x12\x15\n\rerror_message\x18\x01 \x01(\t\x12.\n\x0bretry_after\x18\x02 \x01(\x0b\x32\x19.google.protobuf.Duration\"\xd8\x01\n\x0f\x43ustomHookError\x12\x0e\n\x06stdout\x18\x01 \x01(\x0c\x12\x0e\n\x06stderr\x18\x02 \x01(\x0c\x12\x33\n\thook_type\x18\x03 \x01(\x0e\x32 .gitaly.CustomHookError.HookType\"p\n\x08HookType\x12\x19\n\x15HOOK_TYPE_UNSPECIFIED\x10\x00\x12\x18\n\x14HOOK_TYPE_PRERECEIVE\x10\x01\x12\x14\n\x10HOOK_TYPE_UPDATE\x10\x02\x12\x19\n\x15HOOK_TYPE_POSTRECEIVE\x10\x03\"\xf8\x01\n\tPathError\x12\x0c\n\x04path\x18\x01 \x01(\x0c\x12/\n\nerror_type\x18\x02 \x01(\x0e\x32\x1b.gitaly.PathError.ErrorType\"\xab\x01\n\tErrorType\x12\x1a\n\x16\x45RROR_TYPE_UNSPECIFIED\x10\x00\x12\x19\n\x15\x45RROR_TYPE_EMPTY_PATH\x10\x01\x12/\n+ERROR_TYPE_RELATIVE_PATH_ESCAPES_REPOSITORY\x10\x02\x12\x1c\n\x18\x45RROR_TYPE_ABSOLUTE_PATH\x10\x03\x12\x18\n\x14\x45RROR_TYPE_LONG_PATH\x10\x04\"!\n\x11PathNotFoundError\x12\x0c\n\x04path\x18\x01 \x01(\x0c\",\n\x17\x41mbiguousReferenceError\x12\x11\n\treference\x18\x01 \x01(\x0c\"!\n\x0e\x42\x61\x64ObjectError\x12\x0f\n\x07\x62\x61\x64_oid\x18\x01 \x01(\x0c\"%\n\x14InvalidRevisionRange\x12\r\n\x05range\x18\x01 \x01(\x0c\x42\x34Z2gitlab.com/gitlab-org/gitaly/v16/proto/go/gitalypbb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'errors_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'Z2gitlab.com/gitlab-org/gitaly/v16/proto/go/gitalypb'
  _globals['_ACCESSCHECKERROR']._serialized_start=56
  _globals['_ACCESSCHECKERROR']._serialized_end=149
  _globals['_INDEXERROR']._serialized_start=152
  _globals['_INDEXERROR']._serialized_end=454
  _globals['_INDEXERROR_ERRORTYPE']._serialized_start=231
  _globals['_INDEXERROR_ERRORTYPE']._serialized_end=454
  _globals['_INVALIDREFFORMATERROR']._serialized_start=456
  _globals['_INVALIDREFFORMATERROR']._serialized_end=493
  _globals['_NOTANCESTORERROR']._serialized_start=495
  _globals['_NOTANCESTORERROR']._serialized_end=562
  _globals['_CHANGESALREADYAPPLIEDERROR']._serialized_start=564
  _globals['_CHANGESALREADYAPPLIEDERROR']._serialized_end=592
  _globals['_MERGECONFLICTERROR']._serialized_start=594
  _globals['_MERGECONFLICTERROR']._serialized_end=673
  _globals['_REFERENCESLOCKEDERROR']._serialized_start=675
  _globals['_REFERENCESLOCKEDERROR']._serialized_end=712
  _globals['_REFERENCEEXISTSERROR']._serialized_start=714
  _globals['_REFERENCEEXISTSERROR']._serialized_end=773
  _globals['_REFERENCENOTFOUNDERROR']._serialized_start=775
  _globals['_REFERENCENOTFOUNDERROR']._serialized_end=823
  _globals['_REFERENCESTATEMISMATCHERROR']._serialized_start=825
  _globals['_REFERENCESTATEMISMATCHERROR']._serialized_end=932
  _globals['_REFERENCEUPDATEERROR']._serialized_start=934
  _globals['_REFERENCEUPDATEERROR']._serialized_end=1014
  _globals['_RESOLVEREVISIONERROR']._serialized_start=1016
  _globals['_RESOLVEREVISIONERROR']._serialized_end=1056
  _globals['_LIMITERROR']._serialized_start=1058
  _globals['_LIMITERROR']._serialized_end=1141
  _globals['_CUSTOMHOOKERROR']._serialized_start=1144
  _globals['_CUSTOMHOOKERROR']._serialized_end=1360
  _globals['_CUSTOMHOOKERROR_HOOKTYPE']._serialized_start=1248
  _globals['_CUSTOMHOOKERROR_HOOKTYPE']._serialized_end=1360
  _globals['_PATHERROR']._serialized_start=1363
  _globals['_PATHERROR']._serialized_end=1611
  _globals['_PATHERROR_ERRORTYPE']._serialized_start=1440
  _globals['_PATHERROR_ERRORTYPE']._serialized_end=1611
  _globals['_PATHNOTFOUNDERROR']._serialized_start=1613
  _globals['_PATHNOTFOUNDERROR']._serialized_end=1646
  _globals['_AMBIGUOUSREFERENCEERROR']._serialized_start=1648
  _globals['_AMBIGUOUSREFERENCEERROR']._serialized_end=1692
  _globals['_BADOBJECTERROR']._serialized_start=1694
  _globals['_BADOBJECTERROR']._serialized_end=1727
  _globals['_INVALIDREVISIONRANGE']._serialized_start=1729
  _globals['_INVALIDREVISIONRANGE']._serialized_end=1766
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: mercurial-operations.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from . import errors_pb2 as errors__pb2
from . import lint_pb2 as lint__pb2
from . import shared_pb2 as shared__pb2
from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2
from . import operations_pb2 as operations__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1amercurial-operations.proto\x12\x07hgitaly\x1a\x0c\x65rrors.proto\x1a\nlint.proto\x1a\x0cshared.proto\x1a\x1fgoogle/protobuf/timestamp.proto\x1a\x10operations.proto\"\x94\x01\n\x14MergeAnalysisRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\x12\x17\n\x0fsource_revision\x18\x02 \x01(\x0c\x12\x17\n\x0ftarget_revision\x18\x03 \x01(\x0c\x12\x1c\n\x14skip_conflicts_check\x18\x04 \x01(\x08\"\xad\x02\n\x15MergeAnalysisResponse\x12\x17\n\x0fis_fast_forward\x18\x01 \x01(\x08\x12\x1f\n\x17has_obsolete_changesets\x18\x02 \x01(\x08\x12\x1f\n\x17has_unstable_changesets\x18\x03 \x01(\x08\x12\x15\n\rhas_conflicts\x18\x04 \x01(\x08\x12\x18\n\x10target_is_public\x18\x05 \x01(\x08\x12\x16\n\x0etarget_node_id\x18\x06 \x01(\t\x12\x15\n\rtarget_branch\x18\x07 \x01(\x0c\x12\x14\n\x0ctarget_topic\x18\x08 \x01(\x0c\x12\x16\n\x0esource_node_id\x18\t \x01(\t\x12\x15\n\rsource_branch\x18\n \x01(\x0c\x12\x14\n\x0csource_topic\x18\x0b \x01(\x0c\"\xad\x01\n\x17PublishChangesetRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\x12\x1a\n\x04user\x18\x02 \x01(\x0b\x32\x0c.gitaly.User\x12/\n\x08hg_perms\x18\x03 \x01(\x0e\x32\x1d.hgitaly.MercurialPermissions\x12\x17\n\x0fgitlab_revision\x18\x04 \x01(\x0c\"\x1a\n\x18PublishChangesetResponse\"P\n\x15PublishChangesetError\x12.\n\x0bgitlab_hook\x18\x01 \x01(\x0b\x32\x17.gitaly.CustomHookErrorH\x00\x42\x07\n\x05\x65rror\"\xa1\x02\n\x12MergeBranchRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\x12\x1a\n\x04user\x18\x02 \x01(\x0b\x32\x0c.gitaly.User\x12/\n\x08hg_perms\x18\x03 \x01(\x0e\x32\x1d.hgitaly.MercurialPermissions\x12\x11\n\tcommit_id\x18\x04 \x01(\t\x12\x0e\n\x06\x62ranch\x18\x05 \x01(\x0c\x12\x0f\n\x07message\x18\x06 \x01(\x0c\x12-\n\ttimestamp\x18\x07 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x18\n\x10\x65xpected_old_oid\x18\x08 \x01(\t\x12\x13\n\x0bsemi_linear\x18\t \x01(\x08\"K\n\x13MergeBranchResponse\x12\x34\n\rbranch_update\x18\x01 \x01(\x0b\x32\x1d.gitaly.OperationBranchUpdate\"\xe7\x01\n\x10MergeBranchError\x12.\n\x0bgitlab_hook\x18\x01 \x01(\x0b\x32\x17.gitaly.CustomHookErrorH\x00\x12.\n\x08\x63onflict\x18\x02 \x01(\x0b\x32\x1a.gitaly.MergeConflictErrorH\x00\x12\x37\n\x0freference_check\x18\x03 \x01(\x0b\x32\x1c.gitaly.ReferenceUpdateErrorH\x00\x12\x31\n\tpre_check\x18\x04 \x01(\x0e\x32\x1c.hgitaly.PreCheckUpdateErrorH\x00\x42\x07\n\x05\x65rror\"\x9a\x01\n\rCensorRequest\x12,\n\nrepository\x18\x01 \x01(\x0b\x32\x12.gitaly.RepositoryB\x04\x98\xc6,\x01\x12\x1a\n\x04user\x18\x02 \x01(\x0b\x32\x0c.gitaly.User\x12\x19\n\x11\x63hangeset_node_id\x18\x03 \x01(\t\x12\x11\n\tfile_path\x18\x04 \x01(\x0c\x12\x11\n\ttombstone\x18\x05 \x01(\x0c\"\x10\n\x0e\x43\x65nsorResponse*8\n\x14MercurialPermissions\x12\x08\n\x04READ\x10\x00\x12\t\n\x05WRITE\x10\x01\x12\x0b\n\x07PUBLISH\x10\x02*k\n\x13PreCheckUpdateError\x12\x0e\n\nNO_PROBLEM\x10\x00\x12\x14\n\x10NOT_FAST_FORWARD\x10\x01\x12\x16\n\x12OBSOLETE_CHANGESET\x10\x02\x12\x16\n\x12UNSTABLE_CHANGESET\x10\x03\x32\xea\x02\n\x1aMercurialOperationsService\x12V\n\rMergeAnalysis\x12\x1d.hgitaly.MergeAnalysisRequest\x1a\x1e.hgitaly.MergeAnalysisResponse\"\x06\xfa\x97(\x02\x08\x02\x12_\n\x10PublishChangeset\x12 .hgitaly.PublishChangesetRequest\x1a!.hgitaly.PublishChangesetResponse\"\x06\xfa\x97(\x02\x08\x01\x12\x41\n\x06\x43\x65nsor\x12\x16.hgitaly.CensorRequest\x1a\x17.hgitaly.CensorResponse\"\x06\xfa\x97(\x02\x08\x01\x12P\n\x0bMergeBranch\x12\x1b.hgitaly.MergeBranchRequest\x1a\x1c.hgitaly.MergeBranchResponse\"\x06\xfa\x97(\x02\x08\x01\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'mercurial_operations_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _MERGEANALYSISREQUEST.fields_by_name['repository']._options = None
  _MERGEANALYSISREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _PUBLISHCHANGESETREQUEST.fields_by_name['repository']._options = None
  _PUBLISHCHANGESETREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _MERGEBRANCHREQUEST.fields_by_name['repository']._options = None
  _MERGEBRANCHREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _CENSORREQUEST.fields_by_name['repository']._options = None
  _CENSORREQUEST.fields_by_name['repository']._serialized_options = b'\230\306,\001'
  _MERCURIALOPERATIONSSERVICE.methods_by_name['MergeAnalysis']._options = None
  _MERCURIALOPERATIONSSERVICE.methods_by_name['MergeAnalysis']._serialized_options = b'\372\227(\002\010\002'
  _MERCURIALOPERATIONSSERVICE.methods_by_name['PublishChangeset']._options = None
  _MERCURIALOPERATIONSSERVICE.methods_by_name['PublishChangeset']._serialized_options = b'\372\227(\002\010\001'
  _MERCURIALOPERATIONSSERVICE.methods_by_name['Censor']._options = None
  _MERCURIALOPERATIONSSERVICE.methods_by_name['Censor']._serialized_options = b'\372\227(\002\010\001'
  _MERCURIALOPERATIONSSERVICE.methods_by_name['MergeBranch']._options = None
  _MERCURIALOPERATIONSSERVICE.methods_by_name['MergeBranch']._serialized_options = b'\372\227(\002\010\001'
  _globals['_MERCURIALPERMISSIONS']._serialized_start=1649
  _globals['_MERCURIALPERMISSIONS']._serialized_end=1705
  _globals['_PRECHECKUPDATEERROR']._serialized_start=1707
  _globals['_PRECHECKUPDATEERROR']._serialized_end=1814
  _globals['_MERGEANALYSISREQUEST']._serialized_start=131
  _globals['_MERGEANALYSISREQUEST']._serialized_end=279
  _globals['_MERGEANALYSISRESPONSE']._serialized_start=282
  _globals['_MERGEANALYSISRESPONSE']._serialized_end=583
  _globals['_PUBLISHCHANGESETREQUEST']._serialized_start=586
  _globals['_PUBLISHCHANGESETREQUEST']._serialized_end=759
  _globals['_PUBLISHCHANGESETRESPONSE']._serialized_start=761
  _globals['_PUBLISHCHANGESETRESPONSE']._serialized_end=787
  _globals['_PUBLISHCHANGESETERROR']._serialized_start=789
  _globals['_PUBLISHCHANGESETERROR']._serialized_end=869
  _globals['_MERGEBRANCHREQUEST']._serialized_start=872
  _globals['_MERGEBRANCHREQUEST']._serialized_end=1161
  _globals['_MERGEBRANCHRESPONSE']._serialized_start=1163
  _globals['_MERGEBRANCHRESPONSE']._serialized_end=1238
  _globals['_MERGEBRANCHERROR']._serialized_start=1241
  _globals['_MERGEBRANCHERROR']._serialized_end=1472
  _globals['_CENSORREQUEST']._serialized_start=1475
  _globals['_CENSORREQUEST']._serialized_end=1629
  _globals['_CENSORRESPONSE']._serialized_start=1631
  _globals['_CENSORRESPONSE']._serialized_end=1647
  _globals['_MERCURIALOPERATIONSSERVICE']._serialized_start=1817
  _globals['_MERCURIALOPERATIONSSERVICE']._serialized_end=2179
# @@protoc_insertion_point(module_scope)



# Copyright 2021 Georges Racinet <georges.racinet@octobus.net>
#
# This software may be used and distributed according to the terms of the
# GNU General Public License version 2 or any later version.
#
# SPDX-License-Identifier: GPL-2.0-or-later
import pathlib

TEST_DATA_DIR = pathlib.Path(__file__).parent / 'data'



# Copyright 2020-2022 Georges Racinet <georges.racinet@octobus.net>
#
# This software may be used and distributed according to the terms of the
# GNU General Public License version 2 or any later version.
#
# SPDX-License-Identifier: GPL-2.0-or-later
"""Multiprocess server

The main entry point is :func:`run_forever`
"""
import contextlib
import logging
from multiprocessing import cpu_count
import signal
import socket
import time
from urllib.parse import urlparse

from .address import (
    DEFAULT_TCP_PORT,
    InvalidUrl,
    UnsupportedUrlScheme,
    analyze_netloc,
)
from .mono import (
    BindError,
    server_process,
)
from .worker import WorkerProcess

logger = logging.getLogger(__name__)


class SocketReusePortError(RuntimeError):
    """If the socket could not be flagged with SO_REUSEPORT"""


def prefork_info_from_url(url):
    """Return needed information for prefork processing from URL."""
    try:
        parsed = urlparse(url)
    except ValueError as exc:
        raise InvalidUrl(url, *exc.args)

    scheme = parsed.scheme
    if scheme in ('tcp', 'tls'):
        try:
            family, host, port = analyze_netloc(parsed.netloc)
        except ValueError:
            # can't really happen after urlparse (does already same checks)
            # still catching to be refactor-proof
            raise InvalidUrl(url)

        if port is None:
            port = DEFAULT_TCP_PORT
        return scheme, (family, host, port)
    else:
        return scheme, None


@contextlib.contextmanager
def prebind_sockets(listen_urls):
    """Pre-bind all sockets with the SO_REUSEPORT option."""
    # failing early if there's one invalid URL in the mix, so that
    # we don't have anything to clean up (this would raise ValueError)
    extracted_urls = ((url, prefork_info_from_url(url)) for url in listen_urls)
    prebound_urls = []

    sockets = []

    def close_sockets():
        for sock in sockets:
            sock.close()

    for url, (scheme, info) in extracted_urls:
        try:
            if scheme == 'tcp':
                family, host, port = info

                sock = socket.socket(family, socket.SOCK_STREAM)
                sock.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEPORT, 1)
                if sock.getsockopt(socket.SOL_SOCKET,
                                   socket.SO_REUSEPORT) == 0:
                    raise SocketReusePortError(url)
                try:
                    sock.bind((host, port))
                except Exception:
                    logger.exception("Could not bind on %r (bind info=%r)",
                                     url, info)
                    raise BindError(url)
                else:
                    sockets.append(sock)

                info = ("Pre-bound %r for %r", info[1:], url)
                prebound_urls.append(url)
            elif scheme == 'unix':
                info = ("Unix Domain Socket doesn't need port pre-binding", )
            else:
                raise UnsupportedUrlScheme(scheme)

            logger.info(*info)

        except Exception:
            logger.info("Closing previously bound sockets after exception")
            close_sockets()
            raise

    try:
        yield prebound_urls
    finally:
        close_sockets()


def terminate_workers(witness, workers, sig, *a):
    witness.set(True)
    logger.info("Catched signal %d, workers=%r", sig, workers)
    for worker in workers:
        logger.info("Terminating %s (alive is %r)", worker, worker.is_alive())
        # using is_alive() could be subject to a race, whereas
        # terminate() will send SIGTERM as soon as possible.
        try:
            worker.terminate()
        except Exception as exc:
            # only warning because it may not even be alive.
            logger.exception("Terminating worker %s failed: %r",
                             worker, exc)


class MutableBoolean:
    def __init__(self, v):
        self.set(v)

    def set(self, v):
        self.v = v

    def __bool__(self):
        return self.v


def termination_signals(workers, termination):
    """Signal handler for termination.

    :param workers: iterable of the :class:`Process` instances of the workers.
    :param termination: :class:`MutableBoolean` instance that will be set
       to ``True`` to indicate that worker termination is due to this handler.
       Main use-case would be to avoid restarting them.
    """
    for sig_name in ('SIGTERM', 'SIGINT'):
        sig = getattr(signal, sig_name, None)
        if sig is not None:
            signal.signal(sig, lambda *a: terminate_workers(
                termination, workers, *a))


def run_forever(listen_urls, storages,
                nb_workers=None,
                restart_done_workers=False,
                monitoring_interval=60,
                max_rss_mib=1024,
                mono_process=False,
                **process_kwargs):
    """Run the server, never stopping

    :param listen_urls: list of URLs, given as in the same form as in
       GitLab configuration files.
    :param storages: a :class:`dict`, mapping storage names to the
       corresponding root directories for repositories.
    :param float monitoring_interval: time, in seconds, between successive
       monitorings of workers.
    :param max_rss_mib: maximum Resident Set Size of workers, in MiB. If
       a worker goes over it, it will be gracefully restarted.
    """
    if mono_process:
        return server_process(0, listen_urls, storages,
                              mono_process=True,
                              **process_kwargs)

    if nb_workers is None:
        nb_workers = cpu_count() // 2 + 1

    worker_kwargs = dict(restart_done_workers=restart_done_workers,
                         max_rss=max_rss_mib << 20,
                         )
    worker_kwargs.update(process_kwargs)

    with prebind_sockets(listen_urls) as prebound_urls:
        workers = [WorkerProcess.run(process_callable=server_process,
                                     process_args=(0, listen_urls, storages),
                                     **worker_kwargs)]
        if prebound_urls:
            workers.extend(
                WorkerProcess.run(process_callable=server_process,
                                  process_args=(i, prebound_urls, storages),
                                  **worker_kwargs)
                for i in range(1, nb_workers))
        else:
            logger.info("No socket prebound for multiprocessing "
                        "(expected if listening only to Unix Domain socket) "
                        "Starting only one worker")

        general_shutdown = MutableBoolean(False)
        termination_signals(workers, general_shutdown)

        logger.info("All %d worker processes started", len(workers))

        keep_monitoring = True
        while keep_monitoring and not general_shutdown:
            for worker in workers:
                keep_monitoring &= worker.watch()
            time.sleep(monitoring_interval)

        logger.warning("General shutdown required, wrapping up")
        for worker in workers:
            worker.join()
        logger.info("All worker processes are finished. Closing down")
